extern "C" {

#include "fx.hpp"
#include "tools.hpp"
#include "keyboard.hpp"
#include "mcs.hpp"

#define MAXEDITBUFFER 21

#define cSTATUS 0
#define cWIDTH 1
#define cDELAY 2
#define cCOUNT 3
#define cAUTOOFF 4
#define cDELAY2 5
#define cLASTITEM 5

#define sizeSTATUS 2
#define sizeWIDTH 6
#define sizeDELAY 6
#define sizeCOUNT 8
#define sizeAUTOOFF 4
#define sizeDELAY2 4

#define SETUP "SETUP"

const unsigned char*sCaption[] = { "stat 0/1", "width (ms)", "delay (s)", "count", "autooff (min)", "delay2 (ms)" };
unsigned int parDefault[] = { 1, 50, 5, 20, 1, 0 };
unsigned int parSize[] = { sizeSTATUS, sizeWIDTH, sizeDELAY, sizeCOUNT, sizeAUTOOFF, sizeDELAY2 };

enum TIOmode { modeINPUT, modeOUTPUT };
enum TPhase { phIDLE, phALARM, phMANUALBREAK, phARM };

//
void SetupHardware( TIOmode _mode ){
unsigned int key;
	switch ( GetMPU() ){
		case mpu7705 :
		case mpu7337 :
		case mpu7355 :
// SCIF2 clock on (STBCR3.MSTP31)
			*(unsigned char*)0xA40A0000 &= ~0x02;
// switch off SCSMR_2.TE and SCSMR_2.RE	
			*(unsigned short*)0xA4410008 &= ~0x0030;
// SCIF2 clock off (STBCR3.MSTP31)
			*(unsigned char*)0xA40A0000 |= 0x02;
// set bit 6 of port G to output mode
			*(unsigned short*)0xA400010C = ( *(unsigned short*)0xA400010C & ~0x3000 ) | 0x1000;
// set bit 5 and 6 of port G
			*(unsigned char*)0xA400012C |= 0x60;
			switch ( _mode ){
				case modeOUTPUT:
// set port SC bit 0 to output	
					*(unsigned short*)0xA4000116 = ( *(unsigned short*)0xA4000116 & ~0x0003 ) | 0x0001;
					break;
				case modeINPUT:
// set port SC bit 0 to input	
					*(unsigned short*)0xA4000116 = ( *(unsigned short*)0xA4000116 & ~0x0003 ) | 0x0002;
					break;
			}
			break;
			
		case mpu7305 :
		case mpu7724 :
// SCIF2 clock on (MSTPCR0.MSTP007)	
			*(unsigned int*)0xA4150030 &= ~0x00000080;
// switch off SCSMR_2.TE and SCSMR_2.RE	
			*(unsigned short*)0xA4410008 &= ~0x0030;
// SCIF2 clock off (MSTPCR0.MSTP007)
			*(unsigned int*)0xA4150030 |= 0x00000080;
			
// set bit 3 of port U to output mode
			*(unsigned short*)0xA4050142 = ( *(unsigned short*)0xA4050142 & ~0x00C0 ) | 0x0040;
// set bit 4 and 5 of port U
			*(unsigned char*)0xA4050162 |= 0x0C;
			
			switch( GetMPU() ){
				case mpu7305 :
// set port J bit 2 to output	
					*(unsigned short*)0xA4050110 = ( *(unsigned short*)0xA4050110 & ~0x0030 ) | 0x0010;
// set port J bit 3 to input	
					*(unsigned short*)0xA4050110 = ( *(unsigned short*)0xA4050110 & ~0x00C0 ) | 0x0080;
					break;
					
				case mpu7724 :	// only for reference
// set bit 2 of port F to output mode
					*(unsigned short*)0xA405010A = ( *(unsigned short*)0xA405010A & ~0x0030 ) | 0x0010;
// set bit 1 of port M to input mode
					*(unsigned short*)0xA4050116 = ( *(unsigned short*)0xA4050116 & ~0x000C ) | 0x0004;
// set bit 6 of PSELA
					*(unsigned short*)0xA405014E |= 0x0040;
					break;
			}
			break;
	}
}

//
void SetPin( int state ){
unsigned int key;
	switch ( GetMPU() ){
		case mpu7705 :
		case mpu7337 :
		case mpu7355 :
// set/clear port SC bit 0
			if ( state ) *(unsigned char*)0xA4000136 |= 0x01;
			else *(unsigned char*)0xA4000136 &= ~0x01;
			break;
			
		case mpu7305 :
// set/clear port J bit 2
			if ( state ) *(unsigned char*)0xA4050130 |= 0x04;
			else *(unsigned char*)0xA4050130 &= ~0x04;
			break;
			
		case mpu7724 :	// only for reference
// set/clear port F bit 2
			if ( state ) *(unsigned char*)0xA405012A |= 0x04;
			else *(unsigned char*)0xA405012A &= ~0x04;
			break;
	}
}

//
int GetPin( void ){
unsigned int key;
int iresult = 0;
	switch ( GetMPU() ){
		case mpu7705 :
		case mpu7337 :
		case mpu7355 :
// get port SC bit 0
			if ( *(unsigned char*)0xA4000136 & 0x01 ) iresult = 1;
			break;
			
		case mpu7305 :
// get port J bit 2
			if ( *(unsigned char*)0xA4050130 & 0x08 ) iresult = 1;
			break;
			
		case mpu7724 :	// only for reference
// get port F bit 3
			if ( *(unsigned char*)0xA405012A & 0x08 ) iresult = 1;
			break;
	}
	return iresult;
}


//
void ReleaseHardware( void ){
//	Serial_ResetAndDisable();
}

//
void TimeToStr( unsigned char time[8], unsigned char*sresult ){
	strcpy( (char*)sresult, "xx.xx.xxxx xx:xx:xx" );
	//              0123456789012345678 
	ByteToHex( time[ 3 ], sresult );
	ByteToHex( time[ 2 ], sresult+3 );
	ByteToHex( time[ 0 ], sresult+6 );
	ByteToHex( time[ 1 ], sresult+8 );
	ByteToHex( time[ 4 ], sresult+11 );
	ByteToHex( time[ 5 ], sresult+14 );
	ByteToHex( time[ 6 ], sresult+17 );
}

//
TPhase AutoPowerOff( int AutoPowerOffTimeSeconds ){
int iAutoOff, iBreak;
unsigned char s[ MAXEDITBUFFER ];
int iTicks;
TPhase phase = phIDLE;

	Bdisp_AllClr_VRAM();
	locate( 1, 1 );
	Print( "Auto power off..." );
	locate( 1, 3 );
	Print( "Time to off" );
	locate( 1, 6 );
	Print( "Hit any key to" );
	locate( 12, 7 );
	Print( "cancel..." );

	iBreak = 0;
	iAutoOff = AutoPowerOffTimeSeconds;
	while ( iAutoOff ){
		itoa( iAutoOff, s );
		locate( 3, 4 );
		Print( s );
		PrintLine( " seconds", 21 );
		Bdisp_PutDisp_DD();
		iTicks = RTC_GetTicks();
		while ( !RTC_Elapsed_ms( iTicks, 1000 ) ){
			if ( PRGM_GetKey() ){
				phase = phMANUALBREAK;
				iBreak = 1;
				break;
			}
		}
		if ( iBreak ) break;
		iAutoOff--;
	}
	if ( !iBreak ) PowerOff( 1 );
	while( PRGM_GetKey() );
	Keyboard_ClrBuffer();
	
	return phase;
}

unsigned int alarmcount;

//
int Start( TPhase&phase ){
unsigned int key;
int lContinue = 1;
int lRefresh = 1;
int iValues[ cLASTITEM + 1 ];
int iTicks, iCount, iLastTicks, iNullTicks, ix;
unsigned char s[ MAXEDITBUFFER ];

	SetAutoPowerOffFlag( 0 );
	
	SetupHardware( modeOUTPUT );
	
	for ( ix=cSTATUS; ix<=cLASTITEM; ix++ ){
		iValues[ ix ] = parDefault[ ix ];
	}
	ReadMainMemory( 0, SETUP, sizeof( iValues ), &iValues );
	
	if ( iValues[ cSTATUS ] ) SetPin( 0 ); else SetPin( 1 );

	iCount = 0;
	iTicks = 0;
	iLastTicks = 0;
	iNullTicks = RTC_GetTicks()/128;
	while ( lContinue ){
		if ( iCount > iValues[ cCOUNT ] ) break;
	    if ( iLastTicks != RTC_GetTicks()/128 ) lRefresh = 1;
		if ( lRefresh ){
			Bdisp_AllClr_VRAM();
			locate( 1, 1 );
			Print( "D-Serial running..." );
			
			if ( alarmcount ){
				locate( 1, 2 );
				Print( "Alarm: " );
				itoa( alarmcount, s );
				Print( s );
			};
			
			locate( 1, 3 );
			Print( "Count: " );
			locate( 1, 4 );
			Print( "  " );
			itoa( iCount, s );
			Print( s );
			Print( " of " );
			itoa( iValues[ cCOUNT ], s );
			Print( s );
			locate( 1, 5 );
			Print( "Runtime: " );
			locate( 1, 6 );
			Print( "  " );
			iLastTicks = RTC_GetTicks()/128;
			itoa( iLastTicks-iNullTicks, s );
			Print( s );
			Print( " of " );
			itoa( iValues[ cCOUNT ]*iValues[ cDELAY ], s );
			Print( s );
			Print( " s" );
			locate( 1, 8 );
			Print( "EXIT to exit" );
			
			Bdisp_PutDisp_DD();
			lRefresh = 0;
		}
		key = PRGM_GetKey();
		switch ( key ){
			case KEY_PRGM_EXIT :
				phase = phMANUALBREAK;
				lContinue = 0;
				break;
			case 0 :
				if ( RTC_Elapsed_ms( iTicks, iValues[ cDELAY ]*1000 ) ){
					iTicks = RTC_GetTicks();
					if ( iValues[ cDELAY2 ] ){
						if ( phase == phARM ){
							if ( iValues[ cAUTOOFF ] == -3 ){
								SetPin( iValues[ cSTATUS ] );
								Wait_ms( 1 );
								SetPin( !iValues[ cSTATUS ] );
							}
							Wait_ms( iValues[ cDELAY2 ] );
						}
					}
					SetPin( iValues[ cSTATUS ] );
					Wait_ms( iValues[ cWIDTH ] );
					SetPin( !iValues[ cSTATUS ] );
					lRefresh = 1;
					iCount++;
				}
				break;
		}
	}
	
	ReleaseHardware();
	
	while( PRGM_GetKey() );
	Keyboard_ClrBuffer();
	
	SetAutoPowerOffFlag( 1 );
	
	if ( lContinue ){
		switch( iValues[ cAUTOOFF ] ){
			case -1 :
				locate( 1, 7 );
				PrintLine( "Ready!", 21 );
				locate( 1, 8 );
				PrintLine( "Hit any key...", 21 );
				GetKey( &key );
				phase = phMANUALBREAK;
				break;
				
			case -2 :
			case -3 :
				break;
				
			default :
				phase = AutoPowerOff( iValues[ cAUTOOFF ]*60 );
				break;
		};	
	}	
	return 1;
}

//
int Counter(){
unsigned int key;
int lContinue = 1;
int laststate = -1, currentstate;
int count = 0;
unsigned char s[ 22 ];

	SetupHardware( modeINPUT );
	
	Bdisp_AllClr_VRAM();
	locate( 1, 1 );
	Print( "D-Serial counter" );
	locate( 1, 8 );
	Print( "EXIT to exit" );
	
	while ( lContinue ){
		currentstate = GetPin();
		if ( currentstate != laststate ){
			if ( laststate && !currentstate ) count++;
			itoa( count, s );
			locate( 1, 3 );
			Print( "count:" );
			locate( 10, 3 );
			Print( s );
			Bdisp_PutDisp_DD();
			laststate = currentstate;
		}
		key = PRGM_GetKey();
		switch ( key ){
			case KEY_PRGM_EXIT :
				lContinue = 0;
				break;
		}
	}

	ReleaseHardware();
	
	while( PRGM_GetKey() );
	Keyboard_ClrBuffer();
	return 1;
}

#define TIMEPOS 6

//
int ArmAlarm( TPhase&phase ){
unsigned int key;
int lContinue = 1;
int laststate = -1, currentstate;
int count = 0;
unsigned char s[ 22 ];
unsigned char time[8];
int lastseconds, iSeconds = 0;

	phase = phIDLE;

	SetupHardware( modeINPUT );
	
	Bdisp_AllClr_VRAM();
	locate( 1, 1 );
	Print( "D-Serial armed!" );
	locate( 1, 8 );
	Print( "EXIT to exit" );
	
	laststate = GetPin();
	while ( lContinue ){
		currentstate = GetPin();
		if ( currentstate != laststate ){
			if ( laststate && !currentstate ) phase = phALARM;
			laststate = currentstate;
		}
		if ( phase == phALARM ) break;
		key = PRGM_GetKey();
		switch ( key ){
			case KEY_PRGM_EXIT :
				phase = phMANUALBREAK;
				lContinue = 0;
				break;
		}
		
		RTC_GetDateTime( time );
		if ( lastseconds != time[ 6 ] ){
			lastseconds = time[ 6 ];
			
			TimeToStr( time, s );
			locate( 1, TIMEPOS );
			Print( s );
			
			itoa( iSeconds, s ); 
			locate( 1, TIMEPOS-1 ); 
			Print( s );
			PrintLine( " s", 21 );
			
			Bdisp_PutDisp_DD();
			
			iSeconds++;
		}
		
	}

	ReleaseHardware();
	
	while( PRGM_GetKey() );
	Keyboard_ClrBuffer();
	return 1;
}


#define XEDIT 15
#define YEDIT 2

//
int Setup( void ){
unsigned int key;
int lContinue = 1, lEdit, lRefresh = 1;
int x, y, xmax, ymax, xlen, ix;
unsigned char sB0[ sizeSTATUS ], sB1[ sizeWIDTH ], sB2[ sizeDELAY ], sB3[ sizeCOUNT ], sB4[ sizeAUTOOFF ], sB5[ sizeDELAY2 ];
unsigned char*sValues[] = { sB0, sB1, sB2, sB3, sB4, sB5 };
int iValues[ cLASTITEM + 1 ];
unsigned char*p;
	
	x = 0;
	y = 0;
	ymax = cLASTITEM;

	Bdisp_AllClr_VRAM();
	locate( 1, 1 );
	Print( "D-Serial Setup" );
	for ( ix=cSTATUS; ix<=cLASTITEM; ix++ ){ 
		memset( sValues[ ix ], 0 , parSize[ ix ] ); // init edit buffers
		iValues[ ix ] = parDefault[ ix ]; // init parameter values
	}
	ReadMainMemory( 0, SETUP, sizeof( iValues ), &iValues );
	for ( ix=cSTATUS; ix<=cLASTITEM; ix++ ){ 
		itoa( iValues[ ix ], sValues[ ix ] ); // write parameter values to edit buffers
		locate( 1, YEDIT+ix );
		Print( (unsigned char*)sCaption[ ix ] );
		locate( XEDIT-1, YEDIT+ix );
		PrintLine( ":", 21 );
	}
	locate( 1, 8 );
	Print( "EXE:SAVE EXIT:NOSAVE" );
	
	Cursor_SetFlashMode( 1 );
	
	while ( lContinue ){
		if ( lRefresh ){
			for ( ix=cSTATUS; ix<=cLASTITEM; ix++ ){ 
				locate( XEDIT, YEDIT+ix );
				PrintLine( sValues[ ix ], 21 );
			}
			lRefresh = 0;
		}
		locate( x+XEDIT, y+YEDIT );
	
		lEdit = 0;
		GetKey( &key );
		switch ( key ){
			case KEY_CHAR_MINUS :
				if ( y < 4 ) break;
				if ( x == 0 ){
					lEdit = 1;
					key = (unsigned char)'-';
				}
				break;
				
			case '0' : case '1' :
				lEdit = 1;
				break;
				
			case '2' : case '3' :	case '4' :
			case '5' : case '6' : case '7' : case '8' : case '9' :
				if ( y > 0 ) lEdit = 1;
				break;
				
			case KEY_CTRL_LEFT :
				if ( x ) x--;
				break;
				
			case KEY_CTRL_RIGHT :
				p = sValues[ y ];
				xlen = strlen( (char*)p );
				if ( x < parSize[ y ] - 2 ) if ( x < xlen ) x++;
				break;
				
			case KEY_CTRL_DEL :
				if ( y == 0 ) break;
				p = sValues[ y ];
				xlen = strlen( (char*)p );
				if ( xlen == x ){
					if ( x ) x--;
					p[ x ] = 0;
				}else for ( ix = x; ix < xlen; ix++ ) p[ ix ] = p[ ix + 1 ];
				lRefresh = 1;
				break;
				
			case KEY_CTRL_UP :
				if ( y ) y--;
				x = 0;
				break;
				
			case KEY_CTRL_DOWN :
				if ( y < ymax ) y++;
				x = 0;
				break;
				
			case KEY_CTRL_EXE :
				for ( ix=cSTATUS; ix<=cLASTITEM; ix++ ){
					iValues[ ix ] = atoi( (char*)sValues[ ix ]  );
				}	
				WriteMainMemory( 0, SETUP, sizeof( iValues ), &iValues );
				
			case KEY_CTRL_EXIT :
				lContinue = 0;
				break;
		}
		if ( lEdit ){
			xmax = parSize[ y ];
			p = sValues[ y ];
			p[ x ] = key;
			if ( x < xmax - 2 ) x++;
			lRefresh = 1;
		}
	}
	
	Cursor_SetFlashMode( 0 );
	while( PRGM_GetKey() );
	Keyboard_ClrBuffer();
	return 1;
}

//
int Test(){
unsigned int key, lcontinue = 1;
unsigned char s[ MAXEDITBUFFER ];
int result, i, j, laststatus, ix;
int lastimpulswidth, start;	// ms
int iValues[ cLASTITEM + 1 ];

	SetupHardware( modeOUTPUT );	
	
	Bdisp_AllClr_VRAM();
	locate( 1, 1 );
	Print( "D-Serial Test" );
	
	locate( 1, 3 );
	PrintLine( "F1: toggle low/high", 21 );
	locate( 1, 4 );
	PrintLine( "TTL low=<0.8V", 21 );
	locate( 1, 5 );
	PrintLine( "TTL high=>2V", 21 );
	locate( 1, 6 );
	Print( "EXE: send impulse" );
	locate( 3, 7 );
	Print( "impulse (ms): " );
	
	for ( ix=cSTATUS; ix<=cLASTITEM; ix++ ) iValues[ ix ] = parDefault[ ix ];
    ReadMainMemory( 0, SETUP, sizeof( iValues ), &iValues );
	
	laststatus = -1;
	lastimpulswidth = -1;
	while ( lcontinue ) {
		if ( lastimpulswidth != iValues[ cWIDTH ] ){
			itoa( iValues[ cWIDTH ], s );
			locate( 16, 7 );
			PrintLine( s, 21 );
			lastimpulswidth = iValues[ cWIDTH ];
			WriteMainMemory( 0, SETUP, sizeof( iValues ), &iValues );
		}
		if ( laststatus != iValues[ cSTATUS ] ){
			locate( 3, 8 );
			SetPin( !iValues[ cSTATUS ] );
			switch ( iValues[ cSTATUS ] ){
				case 0 :
					PrintLine( "status: low", 21 );
					break;
				case 1 :
					PrintLine( "status: high", 21 );
					break;
			}
			WriteMainMemory( 0, SETUP, sizeof( iValues ), &iValues );
			laststatus = iValues[ cSTATUS ];
		}
	
		GetKey( &key );	
		switch ( key ){
			case KEY_CTRL_F1 :
				iValues[ cSTATUS ] = !iValues[ cSTATUS ];
				break;
			
			case KEY_CTRL_EXE :
				if ( iValues[ cDELAY2 ] ){
					if ( iValues[ cAUTOOFF ] == -3 ){
						SetPin( iValues[ cSTATUS ] );
						Wait_ms( 1 );
						SetPin( !iValues[ cSTATUS ] );
					}
					Wait_ms( iValues[ cDELAY2 ] );
				}
				SetPin( iValues[ cSTATUS ] );
				Wait_ms( iValues[ cWIDTH ] );
				SetPin( !iValues[ cSTATUS ] );
				break;
			
			case KEY_CTRL_DOWN :
				if ( iValues[ cWIDTH ] > 1 ) iValues[ cWIDTH ] -= 1;
				break;
				
			case KEY_CTRL_UP :
				iValues[ cWIDTH ] += 1;
				break;
			
			case KEY_CTRL_LEFT :
				if ( iValues[ cWIDTH ] > 10 ) iValues[ cWIDTH ] -= 10;
				break;
			
			case KEY_CTRL_RIGHT :
				iValues[ cWIDTH ] += 10;
				break;
			
			case KEY_CTRL_EXIT :
				lcontinue = 0;
				break;
			
		};
	};
	ReleaseHardware();
	while( PRGM_GetKey() );
	Keyboard_ClrBuffer();
	
	return 1;
}

//
void Run(){
unsigned char s[ MAXEDITBUFFER ];
unsigned int key, lcontinue = 1;
int lRefresh = 1, w, lastseconds;
unsigned char time[8];
unsigned int iAutoPowerOffTimeInSeconds = GetAutoPowerOffTime()*60;
TPhase phase = phIDLE;
	while ( lcontinue ) {
		if ( lRefresh ) {
			Bdisp_AllClr_VRAM();
			locate( 1, 1 );
			Print( "D-Serial main" );
			locate( 1, 2 );
			Print( "F1: start" );
			locate( 1, 3 );
			Print( "F2: setup" );
			locate( 1, 4 );
			Print( "F3: count" );
			locate( 1, 5 );
			Print( "F4: arm alarm" );
			locate( 1, 6 );
			Print( "F6: debug" );
			
			locate( 15, 1 );
			switch ( GetMPU() ){
				case mpu7337 :
					Print( "SH-7337" );
					break;
				case mpu7355 :
					Print( "SH-7355" );
					break;
				case mpu7305 :
					Print( "SH-7305" );
					break;
				default :
					Print( "SH-xxxx" );
					break;
			}
			
			Bdisp_PutDisp_DD();
			lRefresh = 0;
		}
		switch ( phase ){
			case phALARM :
				alarmcount++;
				key = KEY_PRGM_F1;
				phase = phARM;
				break;
				
			case phARM :
				key = KEY_PRGM_F4;
				break;
				
			default : 
				key = PRGM_GetKey();
				alarmcount = 0;
		}
		
		if ( key != 0 ) iAutoPowerOffTimeInSeconds = GetAutoPowerOffTime()*60;
		switch ( key ){
			case KEY_PRGM_F1 :
				lRefresh = Start( phase );
				break;
			case KEY_PRGM_F2 :
				lRefresh = Setup();
				break;
			case KEY_PRGM_F3 :
				lRefresh = Counter();
				break;
			case KEY_PRGM_F4 :
				lRefresh = ArmAlarm( phase );
				if ( phase == phMANUALBREAK ) phase = phIDLE;
				break;
			case KEY_PRGM_F5 :
				break;
			case KEY_PRGM_F6 :
				lRefresh = Test();
				break;
			case KEY_PRGM_EXIT :
				break;
			case KEY_PRGM_MENU :
				GetKey( &key );
				break;
			case 0 :
				RTC_GetDateTime( time );
				if ( lastseconds != time[ 6 ] ){
					lastseconds = time[ 6 ];
					
					locate( 1, 8 );
					TimeToStr( time, s );
					Print( s );
					
					itoa( iAutoPowerOffTimeInSeconds, s ); 
					locate( 1, 7 ); 
					Print( "Time to off:" ); 
					Print( s );
					PrintLine( " s", 21 );
					
					Bdisp_PutDisp_DD();
					
					if ( iAutoPowerOffTimeInSeconds ) iAutoPowerOffTimeInSeconds--;
				}
				break;
		}
		if ( !iAutoPowerOffTimeInSeconds ){
		    iAutoPowerOffTimeInSeconds = GetAutoPowerOffTime()*60;
			PowerOff( 1 );		
		}
	}
	
}

const unsigned char ctime[] = 
  { 0x20,0x12,0x12,0x01,0x01,0x00,0x00,0x00 };

  
  
//
void GXA_main( void ){
unsigned int key, lSetRTC=0, ticks, w;
unsigned char time[8];
unsigned char s[21];

	alarmcount = 0;

/*
		Bdisp_AllClr_VRAM();
		locate( 1, 1 );
		IntToHex( *(unsigned int*)0xFD800000, s );
		Print( s );
		GetKey( &key );
*/	

//	_mpu = mpuUnknown;
//	_mpux.WORD = 0;
	if ( GetMPU() == mpuUnknown ){
		Bdisp_AllClr_VRAM();
		locate( 1, 1 );
		Print( "MPU not identified!" );
		Bdisp_PutDisp_DD();
		Wait_ms( 1000 );
		locate( 1, 2 );
		Print( "Hit MENU!" );
		while( 1 ) GetKey( &key );
	};
/*	
	if ( GetMPU() == mpu7305 ){
		Bdisp_AllClr_VRAM();
		locate( 1, 1 );
		Print( "GII-2 calculators" );
		locate( 1, 2 );
		Print( "are not supported..." );
		Bdisp_PutDisp_DD();
		Wait_ms( 1000 );
		locate( 1, 3 );
		Print( "...yet!" );
		locate( 1, 5 );
		Print( "Hit MENU!" );
		while( 1 ) GetKey( &key );
	};
*/

// detect uninitialized RTC
	ticks = RTC_GetTicks();
	Wait_ms( 300 );
	lSetRTC = !RTC_Elapsed_ms( ticks, 150 );
	
/*
	lSetRTC = 1;
	RTC_GetDateTime( time );
	while ( 1 ){
		if ( time[ 0 ] != 0x20 ) break;
		w = ( time[ 1 ] >> 4 ) * 10 + time[ 1 ] & 0xF;
		if ( w < 12 ) break;
		w = ( time[ 2 ] >> 4 ) * 10 + time[ 2 ] & 0xF;
		if ( ( w < 1 ) || ( w > 31 ) ) break;
		w = ( time[ 3 ] >> 4 ) * 10 + time[ 3 ] & 0xF;
		if ( ( w < 1 ) || ( w > 12 ) ) break;
		w = ( time[ 4 ] >> 4 ) * 10 + time[ 4 ] & 0xF;
		if ( ( w < 1 ) || ( w > 31 ) ) break;
		w = ( time[ 5 ] >> 4 ) * 10 + time[ 5 ] & 0xF;
		if ( w > 23 ) break;
		w = ( time[ 6 ] >> 4 ) * 10 + time[ 6 ] & 0xF;
		if ( w > 59 ) break;
		w = ( time[ 7 ] >> 4 ) * 10 + time[ 7 ] & 0xF;
		if ( w > 59 ) break;
		lSetRTC = 0;
		break;	
	}
*/	

/*
	RTC_GetDateTime( time );
	locate( 1, 1 );
	IntToHex( time[ 0 ], s );
	Print( s );
	locate( 10, 1 );
	IntToHex( time[ 1 ], s );
	Print( s );
	locate( 1, 2 );
	IntToHex( time[ 2 ], s );
	Print( s );
	locate( 10, 2 );
	IntToHex( time[ 3 ], s );
	Print( s );
	locate( 1, 3 );
	IntToHex( time[ 4 ], s );
	Print( s );
	locate( 10, 3 );
	IntToHex( time[ 5 ], s );
	Print( s );
	locate( 1, 4 );
	IntToHex( time[ 6 ], s );
	Print( s );
	locate( 10, 4 );
	IntToHex( time[ 7 ], s );
	Print( s );
	GetKey( &key );
*/	

	if ( lSetRTC ){
		RTC_SetDateTime( (unsigned char*)ctime );	
		Bdisp_AllClr_VRAM();
		locate( 1, 1 );
		Print( "RTC initialized" );
		locate( 1, 2 );
		Print( "Hit any key..." );
		GetKey( &key );
	}
	
	Run();
}

#pragma section _BR_Size
unsigned long BR_Size;
#pragma section

}

